import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { useSelector, useDispatch } from 'react-redux';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { DEFAULT_VALUES, BOOTSTRAP_TABLE_SORT } from 'common/constants';
import { TABLES, VEHICLE_CERTIFICATION } from 'common/ui-labels';
import { selectCheckedDrivers, selectCheckedVMs, selectDrivers } from '../../store/selectors';
import { openDeleteDriverCertificationModal, selectTableDriver, selectTableDrivers, unselectTableDrivers, openEditDriverCertificationModal } from '../../store/actions';


function btnGroup(singleCertSelected, isDriversSelected, callbackDel, callbackEdit) {
    if (!singleCertSelected) return null;
    return (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="drivers-button-trash"
                onClick={() => {
                    if (isDriversSelected) {
                        callbackDel();
                    }
                }}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="drivers-button-plus"
                onClick={callbackEdit}
            />
        </section>);
}

function DriversTable(props) {
    const { intl } = props;
    const dispatch = useDispatch();
    const drivers = useSelector(selectDrivers);
    const selectedVehModels = useSelector(selectCheckedVMs);
    const selectedDrivers = useSelector(selectCheckedDrivers);
    const tableData = selectedVehModels.length > 0 ? drivers.filter(driver => selectedVehModels.some(vm => driver.vehicleModelsIds.includes(vm.id))) : drivers;
    const singleCertSelected = selectedVehModels.length === 1;
    
    function openDeleteModal() {
        dispatch(openDeleteDriverCertificationModal());
    }
    
    function openEditModal() {
        dispatch(openEditDriverCertificationModal());
    }
    
    const options = {
        btnGroup: () => btnGroup(singleCertSelected, selectedDrivers.length > 0, openDeleteModal, openEditModal),
        searchField: () => <SearchField data-id="drivers-search" placeholder={TABLES.SEARCH(intl)} />,
        noDataText: TABLES.NO_DATA_TO_DISPLAY,
        beforeShowError: () => true,
        defaultSortName: 'name',
        defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
    };
    const headerStyle = {
        marginBottom: '0',
    };
    
    const selectRow = {
        mode: singleCertSelected ? 'checkbox' : null,
        bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
        clickToSelect: !!singleCertSelected, //  default is false
        selected: selectedDrivers.map(cert => cert.obj_uid),
        onSelect: (row) => {
            dispatch(selectTableDriver(row));
        },
        onSelectAll: (isSelected, data) => {
            if (!isSelected) {
                dispatch(unselectTableDrivers(data));
            } else {
                dispatch(selectTableDrivers(data));
            }
        },
    };
    
    return (
        <BootstrapTable
            trClassName={trOddEvenClassFormat}
            tableBodyClass="automation-drivers-table"
            data={tableData}
            options={options}
            selectRow={selectRow}
            headerStyle={headerStyle}
            bodyContainerClass="Admin-BootstrapTable"
            className="middle-align"
            condensed
            search
            hover
        >
            <TableHeaderColumn
                dataField="obj_uid"
                searchable={false}
                hidden
                isKey
            >
                Uid
            </TableHeaderColumn>
            <TableHeaderColumn
                dataField="name"
                tdAttr={{ 'data-id': 'driver-name' }}
                width="170"
                headerAlign="center"
                dataSort
            >
                {VEHICLE_CERTIFICATION.NAME}
            </TableHeaderColumn>
            <TableHeaderColumn
                dataField="vehicleModelsNames"
                tdAttr={{ 'data-id': 'driver-certification' }}
                dataAlign="center"
                width="120"
                headerAlign="center"
                dataSort
            >
                {VEHICLE_CERTIFICATION.MODEL}
            </TableHeaderColumn>
        </BootstrapTable>
    );
}

DriversTable.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

DriversTable.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
};

export default injectIntl(DriversTable);
