import React, { useEffect } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { useSelector, useDispatch } from 'react-redux';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { DEFAULT_VALUES, BOOTSTRAP_TABLE_SORT } from 'common/constants';
import { TABLES, VEHICLE_CERTIFICATION } from 'common/ui-labels';
import { CERT_DEFAULT_VALUES } from '../common/constants';
import { selectVehicleModels, selectCheckedVMs } from '../store/selectors';
import {
    loadDriversCertifications, loadVehicleModels, loadVehicleCertifications, selectTableVehicleModel, selectTableVehicleModels, unselectTableVehicleModels,
    openDeleteCertModal, clearSelected, openEditCertModal,
} from '../store/actions';


function btnGroup(totalCount, selectedVehModels, callbackDel, callbackEdit, intl) {
    const disableAddBtn = totalCount >= CERT_DEFAULT_VALUES.MAX_MODELS;
    return (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="classes-button-trash"
                onClick={() => {
                    if (selectedVehModels.length > 0) {
                        callbackDel(selectedVehModels);
                    }
                }}
            />
            <button
                className={`unstyled-button ${disableAddBtn ? 'disabled' : ''}`}
                data-id="classes-button-plus"
                onClick={callbackEdit}
                disabled={disableAddBtn}
                title={VEHICLE_CERTIFICATION.MAX_LIMIT(intl, CERT_DEFAULT_VALUES.MAX_MODELS)}
            >
                <i className="fa fa-plus fa-lg table-action-icon" />
            </button>
        </section>);
}

function getActions(row, callbackEdit, callbackDel) {
    return (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="classes-actions-edit"
                onClick={() => { callbackEdit(row); }}
                title="Edit"
                style={{ margin: '5px', cursor: 'pointer' }}
                className="fa fa-pencil-alt"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="groups-actions-remove"
                onClick={() => { callbackDel([row]); }}
                title="Delete"
                style={{ margin: '5px', cursor: 'pointer' }}
                className="fa fa-times"
                aria-hidden="true"
            />
        </section>
    );
}

function VehicleModelsTable(props) {
    const { intl } = props;
    const dispatch = useDispatch();
    const vehicleModels = useSelector(selectVehicleModels);
    const selectedVehModels = useSelector(selectCheckedVMs);
    const totalCount = vehicleModels.length;
    
    useEffect(() => {
        dispatch(loadVehicleModels());
        dispatch(loadVehicleCertifications());
        dispatch(loadDriversCertifications());
    }, []);
    
    function openDeleteModal(deletedVehModel) {
        dispatch(openDeleteCertModal(deletedVehModel));
    }
    function openEditModal(vehicleModel) {
        dispatch(openEditCertModal(vehicleModel));
    }
    
    const options = {
        btnGroup: () => btnGroup(totalCount, selectedVehModels, openDeleteModal, openEditModal, intl),
        searchField: () => <SearchField data-id="classes-search" placeholder={TABLES.SEARCH(intl)} />,
        noDataText: TABLES.NO_DATA_TO_DISPLAY,
        beforeShowError: () => true,
        defaultSortName: 'name',
        defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
    };
    const headerStyle = {
        marginBottom: '0',
    };

    const selectRow = {
        mode: 'checkbox',
        bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
        clickToSelect: true, // click to select, default is false
        selected: selectedVehModels.map(vm => vm.id),
        onSelect: (row, isSelected, e) => {
            if (e.target.getAttribute('data-info') === 'actions-section') {
                return false;
            }
            dispatch(selectTableVehicleModel(row));
            dispatch(clearSelected());
        },
        onSelectAll: (isSelected, tableData) => {
            if (!isSelected) {
                dispatch(unselectTableVehicleModels(tableData));
            } else {
                dispatch(selectTableVehicleModels(tableData));
            }
            dispatch(clearSelected());
        },
    };
    
    return (
        <div className="tab-for-admin-groups">
            <h3 className="text-center AdminTab-Title">{VEHICLE_CERTIFICATION.VEHICLE_MODELS}</h3>
            <h4
                className="text-center AdminTab-Subtitle"
                data-id="classes-total-count"
            >
                {VEHICLE_CERTIFICATION.TOTAL_COUNT_FUNC(`${totalCount}/${CERT_DEFAULT_VALUES.MAX_MODELS}`)}
            </h4>
            <hr />
            <BootstrapTable
                trClassName={trOddEvenClassFormat}
                tableBodyClass="automation-classes-table"
                data={vehicleModels}
                options={options}
                selectRow={selectRow}
                headerStyle={headerStyle}
                bodyContainerClass="Admin-BootstrapTable"
                className="middle-align"
                condensed
                search
                hover
            >
                <TableHeaderColumn
                    dataField="id"
                    searchable={false}
                    hidden
                    isKey
                >
                    Id
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="name"
                    tdAttr={{ 'data-id': 'cert-class-name' }}
                    width="140"
                    headerAlign="center"
                    dataSort
                >
                    {VEHICLE_CERTIFICATION.NAME}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataFormat={(cell, row) => (row.certification_required ? VEHICLE_CERTIFICATION.YES : VEHICLE_CERTIFICATION.NO)}
                    tdAttr={{ 'data-id': 'certification-required' }}
                    dataField="certification_required"
                    searchable={false}
                    dataAlign="center"
                    width="100"
                    headerAlign="center"
                    dataSort
                >
                    {VEHICLE_CERTIFICATION.CERTIFICATION}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="vehiclesCount"
                    tdAttr={{ 'data-id': 'vehicles-count' }}
                    searchable={false}
                    dataSort={false}
                    dataAlign="center"
                    width="100"
                    headerAlign="center"
                >
                    {VEHICLE_CERTIFICATION.VEHICLES}
                </TableHeaderColumn>
                <TableHeaderColumn
                    dataField="driversCount"
                    tdAttr={{ 'data-id': 'drivers-count' }}
                    searchable={false}
                    dataAlign="center"
                    width="100"
                    headerAlign="center"
                >
                    {VEHICLE_CERTIFICATION.LICENSED_DRIVERS}
                </TableHeaderColumn>
                <TableHeaderColumn
                    width="50"
                    headerAlign="center"
                    dataAlign="center"
                    expandable={false}
                    dataFormat={(cell, row) => getActions(row, openEditModal, openDeleteModal)}
                >
                    {VEHICLE_CERTIFICATION.ACTIONS}
                </TableHeaderColumn>
            </BootstrapTable>
        </div>
    );
}

VehicleModelsTable.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

VehicleModelsTable.defaultProps = {
    intl: {
        formatMessage: () => {},
    },
};

export default injectIntl(VehicleModelsTable);
