const error = require('http-errors');
const { CertificationRepo } = require('../repository/certification');


async function getVehicleModels({ company_uid, project_uid }) {
    const result = await CertificationRepo.findAllVehicleModels(company_uid, project_uid);
    return {
        vehicle_models: (result || []).map(row => ({
            name: row.name,
            id: row.id,
            certification_required: !!row.certification_required,
        })),
    };
}

async function getDriversVehiclesByModelId(project_uid, model_id) {
    const drivers_list = await CertificationRepo.findAllDriversLicenses({ project_uid });
    const vehicles_list = await CertificationRepo.findAllVehCert({ project_uid });
    const { driver_licenses } = prepareLicensesResult(drivers_list);

    const drivers = driver_licenses.reduce((memo, obj_item) => {
        const license = obj_item.licenses.find(item => item.id === model_id);
        if (license) {
            memo.push({ obj_uid: obj_item.obj_uid, licenses: obj_item.licenses });
        }
        return memo;
    }, []);
    const vehicles = vehicles_list.reduce((memo, vehicle_cert) => {
        if (vehicle_cert.id === model_id) {
            memo.push({ ...vehicle_cert });
        }
        return memo;
    }, []);

    return { drivers, vehicles };
}

async function updateVehicleModel({ certification_required, project_uid, name, id }) {
    const vehicle_model = await CertificationRepo.findVehicleModelById(id, project_uid);
    if (!vehicle_model) {
        throw error(404, `Model "${id}" not found`);
    }

    await CertificationRepo.updateVehicleModel({ name, project_uid, id, certification_required });
    const vehicle_model_updated = await CertificationRepo.findVehicleModelByName(name, project_uid);

    if (vehicle_model_updated) {
        const result = {
            vehicle_model: {
                name: vehicle_model_updated.name,
                id: vehicle_model_updated.id,
                certification_required: !!vehicle_model_updated.certification_required,
            },
            drivers: [],
            vehicles: [],
        };

        if (!!vehicle_model.certification_required !== certification_required) {
            const { drivers, vehicles } = await getDriversVehiclesByModelId(project_uid, id);
            result.drivers = drivers;
            result.vehicles = vehicles;
        }
        return result;
    }
    throw error(404, `Model "${name}" not found`);
}

async function createVehicleModel({ company_uid, project_uid, name, certification_required }) {
    const vehicleModelsCount = await CertificationRepo.findVehicleModelsCount(project_uid);
    if (vehicleModelsCount >= 31) {
        throw error(422, 'The maximum number of models is equal to 31 per project');
    }

    await CertificationRepo.createVehicleModel({ name, company_uid, project_uid, certification_required });
    const result = await CertificationRepo.findVehicleModelByName(name, project_uid);
    if (result) {
        return {
            name: result.name,
            id: result.id,
            certification_required: !!result.certification_required,
        };
    }
    throw error(404, `Model "${name}" not found`);
}

function prepareLicensesResult(result) {
    if (Array.isArray(result) && result.length > 0) {
        const driver_licenses = result.map((item) => {
            const ids = item.ids.split(',');
            const flags = item.certs.split(',');
            const licenses = item.licenses.split(',').map((name, index) => ({
                name,
                certification_required: !!Number(flags[index]),
                id: Number(ids[index]),
            }));
            return {
                obj_uid: item.obj_uid,
                licenses,
            };
        });
        return {
            driver_licenses,
        };
    }
    return {
        driver_licenses: [],
    };
}

async function deleteVehicleModel({ project_uid, id }) {
    const { vehicle_models } = await getVehicleModels({ project_uid });
    const model_exist = vehicle_models.find(item => item.id === id);
    if (!model_exist) {
        throw error(404, `Model ${id} not found`);
    }
    const { drivers, vehicles } = await getDriversVehiclesByModelId(project_uid, id);
    await CertificationRepo.deleteVehicleModel(id, project_uid);
    const vehicle_notify = vehicles.map(vehicle_cert => ({
        ...vehicle_cert,
        id: vehicle_cert.id === id ? 0 : vehicle_cert.id,
        certification_required: vehicle_cert.id === id ? false : vehicle_cert.certification_required,
    }));
    const drivers_notify = drivers.map(obj_item => ({
        obj_uid: obj_item.obj_uid,
        licenses: obj_item.licenses.filter(item => item.id !== id),
    }));

    await CertificationRepo.updateDriverLicVehicleCert({ id: model_exist.id, project_uid });

    return {
        vehicle_model: model_exist,
        drivers: drivers_notify,
        vehicles: vehicle_notify,
    };
}

async function getDriverLicenses({ company_uid, project_uid }) {
    const result = await CertificationRepo.findAllDriversLicenses({ company_uid, project_uid });
    return prepareLicensesResult(result);
}

async function getVehiclesCertifications({ company_uid, project_uid }) {
    const result = await CertificationRepo.findAllVehCert({ company_uid, project_uid });
    if (Array.isArray(result) && result.length > 0) {
        return {
            vehicle_certifications: result.map(item => ({
                obj_uid: item.obj_uid,
                name: item.name,
                id: item.id,
                certification_required: !!item.certification_required,
            })),
        };
    }
    return {
        vehicle_certifications: [],
    };
}

async function editDriverLicenses({ company_uid, project_uid, licenses, obj_uid }) {
    await CertificationRepo.deleteDriverLicenses(obj_uid);
    if (Array.isArray(licenses)) {
        if (licenses.length > 0) {
            // eslint-disable-next-line no-restricted-syntax
            for (const cert of licenses) {
                await CertificationRepo.addDriverLicenses({ obj_uid, company_uid, project_uid, cert });
            }
        }
    }

    const result = (await CertificationRepo.findDriverLicenses(obj_uid) || []);
    const { driver_licenses } = prepareLicensesResult(result);
    return driver_licenses;
}

async function editVehicleCertification({ company_uid, project_uid, id, obj_uid }) {
    const vehicle_model = await CertificationRepo.findVehicleModelById(id, project_uid);

    if (!vehicle_model && id !== 0) {
        throw error(404, 'Model not found');
    }
    const vehicle_certification = await CertificationRepo.findVehCert(obj_uid);

    if (vehicle_certification) {
        if (id !== vehicle_certification.id) {
            await CertificationRepo.updateVehicleCertification({ id, obj_uid, project_uid });
        }
    } else {
        await CertificationRepo.addVehicleCertification({ obj_uid, project_uid, company_uid, id });
    }

    if (!id) {
        return { obj_uid, id: 0, name: null };
    }

    const updated_vehicle_certification = await CertificationRepo.findVehCert(obj_uid, project_uid);
    if (updated_vehicle_certification) {
        return {
            obj_uid: updated_vehicle_certification.obj_uid,
            name: updated_vehicle_certification.name,
            id: updated_vehicle_certification.id,
            certification_required: !!updated_vehicle_certification.certification_required,
        };
    }
    throw error(404, 'Vehicle not found');
}

module.exports = {
    getVehicleModels,
    createVehicleModel,
    updateVehicleModel,
    deleteVehicleModel,
    getDriverLicenses,
    editDriverLicenses,
    editVehicleCertification,
    getVehiclesCertifications,
};
